function [p,Plast,a,q] = LUlower(A,r)

% Implements inversion algorithm for lower banded matrices
% from Theorem 5.6 in manuscript "Computation of 
% quasiseparable representations of Green matrices".
% The algorithm computes Green generators of the
% inverse matrix. 
% 
% Input: A lower banded matrix to be inverted,
%        r lower bandwidth 
%          (i.e., an upper Hessenberg matrix has r = 1)

% Check size of A
[Nr,Nc] = size(A);
if Nr == Nc
    N = Nr;
else
    disp('Matrix is not square')
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 1: compute generators of LR decomposition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Run algorithm from Theorem 5.1
% 
[L, R] = LRfact(A,r);

% Compute lower Green generators of L^{-1} 
% (Lemma 5.5)
pL = zeros(N-r,r);  % i-th row is pL(i)
dL = zeros(N-r,1);  % i-th entry is dL(i)
aL = zeros(r,r,N-1);% i-th matrix is aL(i)
qL = zeros(r,N-r);  % i-th column is qL(i)
% pL(N-r+1) is an rxr matrix and will be defined last
for k = 1:N-r
    pL(k,:) = L(1,1:r,k);
    dL(k) = L(1,r+1,k);
    aL(:,:,k) = L(2:r+1,1:r,k);
    qL(:,k) = L(2:r+1,r+1,k);
end
pLlast = L(1:r,1:r,N-r+1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: compute lower Green generators of inv(A)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Z = R(N-r+1:N,N-r+1:N);

% 2.1: compute q and a
q = qL;   % N-r generators
a = aL;   % N-r generators

% 2.2: compute p
p = zeros(N-r,r);
Plast = Z\L(1:r,1:r,N-r+1);
P = Plast;
for k = N-r:-1:1
    p(k,:) = (pL(k,:)-R(k,k+1:N)*P*a(:,:,k))/R(k,k);
    P = [p(k,:); P*a(:,:,k)];
end

% end of algorithm
end
