% Test LU-based inversion algorithm for lower banded matrices.
% (Example 5)

% Define matrix A to be inverted
% (strongly regular lower band matrix with half-bandwidth r)
N = 10;
r = 2;
A = triu(rand(N),-r)+r*eye(N); 
%A = toeplitz([2;0.45;0.5; zeros(N-3,1)],[2,0.45,0.5,zeros(1,N-3)]);

% Compute Green generators for inv(A)
[p,Plast,a,q] = LUlower(A,r);

% Recover lower triangular part of computed inverse
B = zeros(N);
B(1,1:r) = p(1,:);
for j=2:N-r
    x=p(j,:);
    for k=j-1:-1:2
        B(j,k+r)=x*q(:,k);
        x=x*a(:,:,k);
    end
    B(j,r+1)=x*q(:,1);
    B(j,1:r)=x*a(:,:,1);
end
y = Plast;
for j = N-r:-1:1
    B(N-r+1:N,j+r) = y*q(:,j);
    y = y*a(:,:,j);
end
B(N-r+1:N,1:r) = y;

% Compare with Ainv
Ainv = inv(A); 
disp("Error for stable case")
disp(norm(tril(B,r-1)-tril(Ainv,r-1)))
    
%%%%%%%%%%%%%%%%%%%%%%%%%
% Now let's modify A
deltaexp = 0:8;
errors = zeros(length(deltaexp),1);
errorc = zeros(length(deltaexp),1);
errorq = zeros(length(deltaexp),1);
for dcount = 1:length(deltaexp)
A(1:3,1:3) = [1 1 1; 2 2+10^(-deltaexp(dcount)) 5; 4 6 8];
% Compute Green generators for inv(A)
[p,Plast,a,q] = LUlower(A,r);

% Recover lower triangular part of computed inverse
B = zeros(N);
B(1,1:r) = p(1,:);
for j=2:N-r
    x=p(j,:);
    for k=j-1:-1:2
        B(j,k+r)=x*q(:,k);
        x=x*a(:,:,k);
    end
    B(j,r+1)=x*q(:,1);
    B(j,1:r)=x*a(:,:,1);
end
y = Plast;
for j = N-r:-1:1
    B(N-r+1:N,j+r) = y*q(:,j);
    y = y*a(:,:,j);
end
B(N-r+1:N,1:r) = y;

% Compare with Ainv
% Ainv = inv(vpa(A,64)); 
Ainv=inv(A);
errors(dcount) = norm(tril(B,r-1)-tril(Ainv,r-1))/norm(Ainv);

% Compute Green generators for inv(A)
[p,Plast,a,q] = QRlower(A,r);

% Recover lower triangular part of computed inverse
B = zeros(N);
B(1,1:r) = p(1,:);
for j=2:N-r
    x=p(j,:);
    for k=j-1:-1:2
        B(j,k+r)=x*q(:,k);
        x=x*a(:,:,k);
    end
    B(j,r+1)=x*q(:,1);
    B(j,1:r)=x*a(:,:,1);
end
y = Plast;
for j = N-r:-1:1
    B(N-r+1:N,j+r) = y*q(:,j);
    y = y*a(:,:,j);
end
B(N-r+1:N,1:r) = y;

% Compare with Ainv
% Ainv = inv(vpa(A,64)); 
Ainv=inv(A);
errorq(dcount) = norm(tril(B,r-1)-tril(Ainv,r-1))/norm(Ainv);

% Try classical LU
[L,U] = myLU(A);
C = U\L;
errorc(dcount) = norm(tril(C,r-1)-tril(Ainv,r-1))/norm(Ainv);
end
