% Test QR-based inversion algorithm for 
% ill-conditioned lower banded matrices.

N = 100;
r = 10;
c_range = 1:14;
n_trial = 10;
err_struct = zeros(length(c_range),n_trial);
err_backslash = zeros(length(c_range),n_trial);
for c = c_range
    disp(c)
    for nt = 1:n_trial
    A = condition(100,5,10^c);
    
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = QRlower(A,r);
    
    % Recover lower triangular part of computed inverse
    B = zeros(N);
    B(1,1:r) = p(1,:);
    for j=2:N-r
        x=p(j,:);
        for k=j-1:-1:2
            B(j,k+r)=x*q(:,k);
            x=x*a(:,:,k);
        end
        B(j,r+1)=x*q(:,1);
        B(j,1:r)=x*a(:,:,1);
    end
    y = Plast;
    for j = N-r:-1:1
        B(N-r+1:N,j+r) = y*q(:,j);
        y = y*a(:,:,j);
    end
    B(N-r+1:N,1:r) = y;
    
    % Compare with Ainv
    Ainvex = inv(vpa(A,64));
    Ainv = inv(A);
    nA = norm(Ainvex);
    %disp(norm(tril(B,r-1)-tril(Ainv,r-1)))
    err_struct(c,nt) = norm(tril(B,r-1)-tril(Ainvex,r-1))/nA;
    err_backslash(c,nt) = norm(tril(Ainv,r-1)-tril(Ainvex,r-1))/nA;
    end
end
err_s = sum(err_struct,2)/n_trial;
err_b = sum(err_backslash,2)/n_trial;
plot(log10(err_b),'c')
hold on
plot(log10(err_s),'m')
