% Comparison on running time
% for structured and classical inversion algorithms.
% Matrices are two-side banded and well-conditioned.

r = 5;
Ntest = 600:50:2500;
nt = 10;  % number of trials for each matrix size

time_sLU = zeros(length(Ntest),nt,1);  % structured LU
%time_sLUr = zeros(length(Ntest),1); % sLU with matrix recovery
time_sQR = zeros(length(Ntest),nt,1);  % structured QR
%time_sQRr = zeros(length(Ntest),1); % sQR with matrix recovery
time_inv = zeros(length(Ntest),nt,1);  % MATLAB inv (LU-based) 
time_sparseinv = zeros(length(Ntest),nt,1);  % sparse inv
time_cQR = zeros(length(Ntest),nt,1);  % classical QR

for hh = 1:length(Ntest)
    N = Ntest(hh);
    disp(N)
    for tcount = 1:nt
    A = tril(triu(rand(N),-r),r) + r*eye(N);
    
    %%%%%%%%%%%%%%%%
    % Structured QR
    %%%%%%%%%%%%%%%%
    tic
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = QRbanded(A,r);
    time_sQR(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%%
    % Matrix recovery
    %%%%%%%%%%%%%%%%%
    % Recover lower triangular part of computed inverse
%     tic
%     B = zeros(N);
%     B(1,1:r) = p(1,:);
%     for j=2:N-r
%         x=p(j,:);
%         for k=j-1:-1:2
%             B(j,k+r)=x*q(:,k);
%             x=x*a(:,:,k);
%         end
%         B(j,r+1)=x*q(:,1);
%         B(j,1:r)=x*a(:,:,1);
%     end
%     y = Plast;
%     for j = N-r:-1:1
%         B(N-r+1:N,j+r) = y*q(:,j);
%         y = y*a(:,:,j);
%     end
%     B(N-r+1:N,1:r) = y;
%     time_sQRr(hh) = toc + time_sQR(hh);
%     
    %%%%%%%%%%%%%%%%
    % Structured LU
    %%%%%%%%%%%%%%%%
    tic
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = LUbanded(A,r);
    time_sLU(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%%
    % Matrix recovery
    %%%%%%%%%%%%%%%%%
    % Recover lower triangular part of computed inverse
%     tic
%     B = zeros(N);
%     B(1,1:r) = p(1,:);
%     for j=2:N-r
%         x=p(j,:);
%         for k=j-1:-1:2
%             B(j,k+r)=x*q(:,k);
%             x=x*a(:,:,k);
%         end
%         B(j,r+1)=x*q(:,1);
%         B(j,1:r)=x*a(:,:,1);
%     end
%     y = Plast;
%     for j = N-r:-1:1
%         B(N-r+1:N,j+r) = y*q(:,j);
%         y = y*a(:,:,j);
%     end
%     B(N-r+1:N,1:r) = y;
%     time_sLUr(hh) = toc + time_sLU(hh);
    
    %%%%%%%%%%%%%%%%
    % MATLAB inv
    %%%%%%%%%%%%%%%%
    tic
    C = inv(A);
    time_inv(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%%%%
    % MATLAB sparse inv
    %%%%%%%%%%%%%%%%%%%
    tic
    As = sparse(A);
    Cs = inv(As);
    time_sparseinv(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%
    % Classical QR
    %%%%%%%%%%%%%%%%
    tic
    [Q,R] = qr(A);
    Cqr = R\Q';
    time_cQR(hh,tcount) = toc;
    
    end
end

av_time_sLU = sum(time_sLU,2)/nt;  % structured LU
av_time_sQR = sum(time_sQR,2)/nt;  % structured QR
av_time_inv = sum(time_inv,2)/nt;  % MATLAB inv (LU-based) 
av_time_sparseinv = sum(time_sparseinv,2)/nt;  % sparse inv
av_time_cQR = sum(time_cQR,2)/nt; % classical QR

figure(1)
hold on
plot(log10(Ntest),log10(av_time_sLU))
plot(log10(Ntest),log10(av_time_sQR))
plot(log10(Ntest),log10(av_time_inv))
plot(log10(Ntest),log10(av_time_sparseinv))
plot(log10(Ntest),log10(av_time_cQR))